# Generated by Django 3.0.14 on 2022-03-13 16:34

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('api_fhir_r4', '0002_auto_20220313_1607'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscriptionnotificationresult',
            name='id',
            field=models.UUIDField(db_column='UUID', default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
        ),
    ]
