ERROR_BAD_STATUS = "Bad value in paymentStatus, should be either 'active', 'cancelled', draft', 'entered-in-error'"
ERROR_BAD_PAYMENT_STATUS = "Bad value in paymentStatus, should be either 'paid' or 'cleared'"
ERROR_BAD_TYPE_REQUEST = "Bad value in type in request, should be either 'bill' or 'invoice'"
