# Generated by Django 3.0.14 on 2022-03-13 16:07

import core.datetimes.ad_datetime
import core.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('api_fhir_r4', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalsubscription',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical subscription'},
        ),
        migrations.CreateModel(
            name='SubscriptionNotificationResult',
            fields=[
                ('id', models.UUIDField(db_column='UUID', default=None, editable=False, primary_key=True, serialize=False)),
                ('notified_successfully', models.BooleanField()),
                ('notification_time', core.fields.DateTimeField(db_column='Expiring', default=core.datetimes.ad_datetime.AdDatetime.now)),
                ('error', models.TextField(default=None, null=True)),
                ('subscription', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications_sent', to='api_fhir_r4.Subscription')),
            ],
            options={
                'db_table': 'tblSubscriptionNotificationResult',
                'managed': True,
            },
        ),
    ]
