from api_fhir_r4.tests.mixin.genericTestMixin import GenericTestMixin
from api_fhir_r4.tests.mixin.locationTestMixin import LocationTestMixin
from api_fhir_r4.tests.mixin.operationOutcomeTestMixin import OperationOutcomeTestMixin
from api_fhir_r4.tests.mixin.patientTestMixin import PatientTestMixin
from api_fhir_r4.tests.mixin.claimAdminPractitionerTestMixin import ClaimAdminPractitionerTestMixin
from api_fhir_r4.tests.mixin.claimAdminPractitionerRoleTestMixin import ClaimAdminPractitionerRoleTestMixin
from api_fhir_r4.tests.mixin.claimTestMixin import ClaimTestMixin
from api_fhir_r4.tests.mixin.claimResponseTestMixin import ClaimResponseTestMixin
from api_fhir_r4.tests.mixin.activityDefinitionTestMixin import ActivityDefinitionTestMixin
from api_fhir_r4.tests.mixin.coverageEligibilityRequestTestMixin import CoverageEligibilityRequestTestMixin
from api_fhir_r4.tests.mixin.communicationRequestTestMixin import CommunicationRequestTestMixin
from api_fhir_r4.tests.mixin.communicationTestMixin import CommunicationTestMixin
from api_fhir_r4.tests.mixin.genericFhirAPITestMixin import GenericFhirAPITestMixin
from api_fhir_r4.tests.mixin.fhirApiReadTestMixin import FhirApiReadTestMixin
from api_fhir_r4.tests.mixin.fhirApiCreateTestMixin import FhirApiCreateTestMixin
from api_fhir_r4.tests.mixin.fhirApiUpdateTestMixin import FhirApiUpdateTestMixin
from api_fhir_r4.tests.mixin.fhirApiDeleteTestMixin import FhirApiDeleteTestMixin
from api_fhir_r4.tests.mixin.insurancePlanTestMixin import InsurancePlanTestMixin
from api_fhir_r4.tests.mixin.medicationTestMixin import MedicationTestMixin
from api_fhir_r4.tests.mixin.contractTestMixin import ContractTestMixin
from api_fhir_r4.tests.mixin.groupTestMixin import GroupTestMixin
from api_fhir_r4.tests.mixin.enrolmentOfficerPractitionerTestMixin import EnrolmentOfficerPractitionerTestMixin
from api_fhir_r4.tests.mixin.enrolmentOfficerPractitionerRoleTestMixin import EnrolmentOfficerPractitionerRoleTestMixin
from api_fhir_r4.tests.mixin.coverageTestMixin import CoverageTestMixin
from api_fhir_r4.tests.mixin.paymentNoticeTestMixin import PaymentNoticeTestMixin
from api_fhir_r4.tests.testCodeSystem import *
